@@ -1,6 +1,9 @@
 package Class::InsideOut;
+use strict;
 
-$VERSION     = '1.08';
+use vars qw/$VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS/;
+
+$VERSION     = '1.10';
 @ISA         = qw ( Exporter );
 @EXPORT      = qw ( ); # nothing by default
 @EXPORT_OK   = qw ( new id options private property public readonly register );
@@ -11,19 +14,19 @@ $VERSION     = '1.08';
     "singleton" => [], # just a flag for import()
 );
 
-use strict;
 use Carp;
 use Exporter;
 use Class::ISA;
 use Scalar::Util qw( refaddr reftype blessed );
 
 # Check for XS Scalar::Util with weaken() or warn and fallback
+# syntax of error changed in Scalar::Util so we check both versions
 BEGIN {
     eval { Scalar::Util->import( "weaken" ) };
-    if ( $@ =~ /\AWeak references/ ) {
+    if ( $@ =~ /\AWeak references|weaken is only available/ ) {
         warn "Scalar::Util::weaken unavailable: "
-           . "Class::InsideOut will not be thread-safe\n";
-        *weaken = sub { shift };
+           . "Class::InsideOut will not be thread-safe and will leak memory\n";
+        *weaken = sub { return @_ };
     }
 }
 
@@ -76,7 +79,8 @@ sub import {
         *{ "$caller\::STORABLE_freeze" } = _gen_STORABLE_freeze( $caller, 0 );
         *{ "$caller\::STORABLE_thaw" } = _gen_STORABLE_thaw( $caller );
     }
-    goto &Exporter::import;
+    local $Exporter::ExportLevel = $Exporter::ExportLevel + 1;
+    &Exporter::import;
 }
 
 BEGIN { *id = \&Scalar::Util::refaddr; }
@@ -115,26 +119,26 @@ sub new {
     return $self;
 }
 
-sub private($\%;$) {
+sub private($\%;$) { ## no critic -- prototype
     &_check_property;
     $_[2] ||= {};
     $_[2] = { %{$_[2]}, privacy => 'private' };
     goto &_install_property;
 }
 
-sub property($\%;$) {
+sub property($\%;$) { ## no critic -- prototype
     &_check_property;
     goto &_install_property;
 }
 
-sub public($\%;$) {
+sub public($\%;$) { ## no critic -- prototype
     &_check_property;
     $_[2] ||= {};
     $_[2] = { %{$_[2]}, privacy => 'public' };
     goto &_install_property;
 }
 
-sub readonly($\%;$) {
+sub readonly($\%;$) { ## no critic -- prototype
     &_check_property;
     $_[2] ||= {};
     $_[2] = { 
@@ -175,7 +179,6 @@ sub register {
     return $obj;
 }
 
-
 #--------------------------------------------------------------------------#
 # private functions for implementation
 #--------------------------------------------------------------------------#
@@ -206,7 +209,7 @@ sub CLONE {
 
         # update the registry to the new, cloned object
         weaken ( $OBJECT_REGISTRY{ $new_id } = $object );
-        delete $OBJECT_REGISTRY{ $old_id };
+        _deregister( $old_id );
     }
 }
 
@@ -232,7 +235,7 @@ sub _check_property {
     my ($label, $hash, $opt) = @_;
     local $Carp::CarpLevel = $Carp::CarpLevel + 1;
     croak "Invalid property name '$label': must be a perl identifier"
-        if $label !~ /\A[a-z_]\w*\z/;
+        if $label !~ /\A[a-z_]\w*\z/i;
     croak "Duplicate property name '$label'"
         if grep { $_ eq $label } keys %{ $PROP_DATA_FOR{ caller(1) } }; 
     _check_options( $opt ) if defined $opt;
@@ -245,6 +248,14 @@ sub _class_tree {
     return @{ $CLASS_ISA{ $class } };
 }
 
+# take either object or object id
+sub _deregister {
+    my ($arg) = @_;
+    my $obj_id = ref $arg ? refaddr $arg : $arg;
+    delete $OBJECT_REGISTRY{ $obj_id };
+    return;
+}
+
 # turn object into hash -- see _revert()
 sub _evert {
     my ( $obj ) = @_;
@@ -348,7 +359,7 @@ sub _gen_DESTROY {
         # it should only call SUPER::DEMOLISH if need be; still,
         # rest of the destructor doesn't need the registry, so early deletion
         # by a subclass should be safe
-        delete $OBJECT_REGISTRY{ $obj_id };
+        _deregister( $obj );
 
         return;
     };
@@ -671,7 +682,17 @@ Property accessors may also be hand-written by declaring the property
 Hand-written accessors will be very slightly faster as generated accessors hold
 a reference to the property hash rather than accessing the property hash
 directly.
- 
+
+It is also possible to use a package hash instead of a lexical hash to store
+object properties:
+
+ public name => our %name;
+
+However, this makes private object data accessable outside the class and incurs
+a slight performance penalty when accessing the property hash directly; it is
+not recommended to do this unless you really need it for some specialized
+reason.
+
 == Object construction
 
 {Class::InsideOut} provides no default constructor method as there are many
@@ -777,6 +798,10 @@ on request.  When called as a class method, it returns a blessed anonymous
 scalar.  Arguments will be used to initialize all matching inside-out class
 properties in the {@ISA} tree.  The argument may be a hash or hash reference.
 
+Note: Properties are set directly, not via accessors.  This means {set_hook} 
+functions will not be called.  For more robust argument checking, you will
+need to implement your own constructor.
+
 == {options}
 
  Class::InsideOut::options( \%new_options );
@@ -890,6 +915,11 @@ Programmers seeking a more full-featured approach to inside-out objects are
 encouraged to explore [Object::InsideOut].  Other implementations are also
 noted in [Class::InsideOut::Manual::About].
 
+= KNOWN LIMITATIONS
+
+Requires weak reference support (Perl >= 5.6) and Scalar::Util::weaken() to
+avoid memory leaks and to provide thread-safety.
+
 = ROADMAP
 
 Features slated for after the 1.0 release include:
